@@ -1,6 +1,10 @@
-# Copyright 2003-2011 Daniel Browning <db@kavod.com>. All rights reserved.
-# This program is free software; you may redistribute it and/or modify it
-# under the same terms as Perl itself. See LICENSE for more info.
+# Business::Shipping - Rates and tracking for UPS and USPS
+#
+# $Id: Shipping.pm 299 2005-09-17 01:26:48Z db-ship $
+#
+# Copyright (c) 2003-2005 Daniel Browning <db@kavod.com>. All rights reserved.
+# This program is free software; you may redistribute it and/or modify it under
+# the same terms as Perl itself. See doc/License for more info.
 
 package Business::Shipping;
 
@@ -8,20 +12,13 @@ package Business::Shipping;
 
 Business::Shipping - Rates and tracking for UPS and USPS
 
-=cut
-
-use Any::Moose;
-use Carp;
-use Business::Shipping::Logging;
-use Business::Shipping::Util 'unique';
-
 =head1 VERSION
 
-Version 2.4.1
+Version 2.01
 
 =cut
 
-use version; our $VERSION = qv('2.4.1');
+$VERSION = '2.01';
 
 =head1 SYNOPSIS
 
@@ -49,19 +46,17 @@ Business::Shipping currently supports three shippers:
 
 =over 4
 
-=item * Shipment rate estimation using offline tables. 
-
-As of January, 2007, UPS has only released the data tables in binary Excel 
-format. These have not been converted to the text files necessary for use in 
-this module. A script is distributed with the module for automatically 
-updating the fuel surcharge every month.
+=item * Shipment rate estimation using offline tables.  New tables are released whenever UPS updates them, and
+a script is available for automatically updating the fuel surcharge every month.
 
 =back
 
-=head2 UPS_Online: United Parcel Service using UPS OnLine Tools
+=head2 UPS_Online: United Parcel Service using UPS OnLine Tools (disabled)
 
 =over 4
 
+=item * Disabled as of version 1.90, see doc/UPS_Online_disabled.txt.
+
 =item * Shipment rate estimation
 
 =item * Shipment tracking.
@@ -100,15 +95,13 @@ Add these options to your rate request for C.O.D.:
 
 cod: enable C.O.D.
 
-cod_funds_code:  The code that indicates the type of funds that will be used for
-the COD payment. Required if CODCode is 1, 2, or 3.  Valid Values: 0 = All Funds
-Allowed.  8 = cashier's check or money order, no cash allowed.
+cod_funds_code:  The code that indicates the type of funds that will be used for the COD payment.  
+Required if CODCode is 1, 2, or 3.  Valid Values: 0 = All Funds Allowed.  8 = cashier's check or 
+money order, no cash allowed.
 
-cod_value: The COD value for the package.  Required if COD option is present. 
-Valid values: 0.01 - 50000.00
+cod_value: The COD value for the package.  Required if COD option is present.  Valid values: 0.01 - 50000.00
 
-cod_code: The code associated with the type of COD.  Values: 1 = Regular COD, 
-2 = Express COD, 3 = Tagless COD
+cod_code: The code associated with the type of COD.  Values: 1 = Regular COD, 2 = Express COD, 3 = Tagless COD
  
 For example:
 
@@ -136,27 +129,24 @@ See doc/INSTALL.
 
 =head1 REQUIRED MODULES
 
-See INSTALL.
+The following modules are required for offline UPS rate estimation.  See doc/INSTALL.
 
- Any::Moose (any)
+ Business::Shipping::DataFiles (any)
+ Class::MethodMaker::Engine (any)
  Config::IniFiles (any)
  Log::Log4perl (any)
 
 =head1 OPTIONAL MODULES
 
-For UPS offline rate estimation:
-
- Business::Shipping::DataFiles (any)
-
-The following modules are used by online rate estimation and tracking.  See 
-INSTALL.
+The following modules are used by online rate estimation and tracking.  See doc/INSTALL.
 
  Cache::FileCache (any)
+ Clone (any)
  Crypt::SSLeay (any)
  LWP::UserAgent (any)
  XML::DOM (any)
  XML::Simple (2.05)
-
+ 
 =head1 GETTING STARTED
 
 Be careful to read, understand, and comply with the terms of use for the 
@@ -179,13 +169,12 @@ every first monday of the month.
 =item * Read the legal terms and conditions: 
 L<http://www.ups.com/content/us/en/resources/service/terms/index.html>
 
-=item * L<https://www.ups.com/one-to-one/register>
+=item * L<https://www.ups.com/servlet/registration>
 
 =item * After receiving a User Id and Password from UPS, login, then select
         "Get Access Key", then "Get XML Access Key".
 
-=item * Read more about UPS Online Tools: 
-L<http://www.ups.com/e_comm_access/toolintro>
+=item * Read more about UPS Online Tools at L<http://www.ec.ups.com>
 
 =back
 
@@ -193,9 +182,9 @@ L<http://www.ups.com/e_comm_access/toolintro>
 
 =over 4
 
-=item * L<https://secure.shippingapis.com/Registration/>
+=item * L<http://www.uspswebtools.com/registration/>
 
-=item * (More info at L<http://www.usps.com/webtools/>)
+=item * (More info at L<http://www.uspswebtools.com>)
 
 =item * The online signup will result in a testing-only account (only a small
         sample of queries will work).  
@@ -209,10 +198,9 @@ L<http://www.ups.com/e_comm_access/toolintro>
 
 =head1 ERROR/DEBUG HANDLING
 
-Log4perl is used for logging error, debug, etc. messages. For simple 
-manipulation of the current log level, use the Business::Shipping->log_level()
-class method (below). For more advanced logging/debugging options, see 
-config/log4perl.conf.
+Log4perl is used for logging error, debug, etc. messages.  For simple manipulation of the current log level, 
+use the Business::Shipping->log_level( $log_level ) class method (below).  For more advanced logging/debugging
+options, see config/log4perl.conf.
 
 =head1 Preloading Modules
 
@@ -230,8 +218,9 @@ when preloading is advantagous.  For example:
 
 =over 4
 
-=item * For mod_perl, to load the modules only once at startup to reduce memory
-utilization.
+=item * For mod_perl, to load the modules only once at startup instead of at 
+ startup and then additional modules later on.  (Thanks to Chris Ochs 
+ <chris@paymentonline.com> for contributing to this information).
 
 =item * For compatibilty with some security modules (e.g. Safe).
 
@@ -245,49 +234,70 @@ utilization.
 
 =cut
 
-has 'tx_type'         => (is => 'rw', isa => 'Str');
-has 'shipper'         => (is => 'rw', isa => 'Str');
-has '_user_error_msg' => (is => 'rw', isa => 'Str');
-
-__PACKAGE__->meta()->make_immutable();
+use strict;
+use warnings;
+use Carp;
+use Business::Shipping::Logging;
+use Business::Shipping::Util 'unique';
+#use Business::Shipping::ClassAttribs;
+use Class::MethodMaker 2.0
+    [ 
+      new    => [ qw/ -hash new /                                     ],
+      scalar => [ 'tx_type', 'shipper', '_user_error_msg'             ],
+    ];
 
 $Business::Shipping::RuntimeLoad = 1;
 
-sub import {
-    my ($class_name, $record) = @_;
-
-    return unless defined $record and ref($record) eq 'HASH';
-
-    while (my ($key, $val) = each %$record) {
-        if (lc $key eq 'preload') {
-
+sub import 
+{
+    my ( $class_name, $record ) = @_;
+    
+    return unless defined $record and ref( $record ) eq 'HASH';
+    
+    while ( my ( $key, $val ) = each %$record ) {
+        if ( lc $key eq 'preload' ) {
+            
             # Required modules lists
             # ======================
-            # Each of these modules does a compile-time require of all
+            # Each of these modules does a compile-time require of all 
             # the modules that it needs.  If, in the future, any of these
             # modules switch to a run-time require, then update this list with
             # the modules that may be run-time required.
+            
             my $module_list = {
-                'USPS_Online' =>
-                    ['Business::Shipping::USPS_Online::Tracking',],
-                'UPS_Online' => ['Business::Shipping::UPS_Online::Tracking',],
-                'UPS_Offline' => [],
+                'USPS_Online' => [
+                    'Business::Shipping::USPS_Online::Tracking',
+                ],
+                'UPS_Online' => [
+                    'Business::Shipping::UPS_Online::Tracking',
+                ],
+                'UPS_Offline' => [
+                ],
             };
-
+                    
             my @to_load;
-            while (my ($shipper, $mod_list) = each %$module_list) {
-                if (lc $val eq lc $shipper or lc $val eq 'all') {
-                    my $rate_req_mod
-                        = 'Business::Shipping::' . $shipper . '::RateRequest';
-                    push @to_load, (@$mod_list, $rate_req_mod);
+                
+            if ( lc $val eq 'all' ) {
+                for ( keys %$module_list ) {
+                    my $aryref = $module_list->{ $_ };
+                    push @to_load, @$aryref;
                 }
             }
-
-            if (@to_load) {
-                $Business::Shipping::RuntimeLoad = 0;
+            else {
+                while ( my ( $shipper, $mod_list ) = each %$module_list ) {
+                    if ( lc $val eq lc $shipper ) {
+                        push @to_load, ( 
+                            @$mod_list, 
+                            'Business::Shipping::' . $shipper . '::RateRequest',
+                        );
+                    }
+                }
             }
-            my @unique_to_load = Business::Shipping::Util::unique(@to_load);
-            foreach my $module (@unique_to_load) {
+            
+            if ( @to_load ) 
+                { $Business::Shipping::RuntimeLoad = 0 };
+            
+            foreach my $module ( Business::Shipping::Util::unique( @to_load ) ) {
                 eval "use $module;";
                 die $@ if $@;
             }
@@ -301,15 +311,16 @@ Generic attribute setter.
 
 =cut
 
-sub init {
-    my ($self, %args) = @_;
-
-    foreach my $arg (keys %args) {
-        if ($self->can($arg)) {
-            $self->$arg($args{$arg});
+sub init
+{
+    my ( $self, %args ) = @_;
+    
+    foreach my $arg ( keys %args ) {
+        if ( $self->can( $arg ) ) {
+            $self->$arg( $args{ $arg } );
         }
     }
-
+    
     return;
 }
 
@@ -319,14 +330,29 @@ Log and store errors that should be visibile to the user.
 
 =cut
 
-sub user_error {
-    my ($self, $msg) = @_;
+sub user_error
+{
+    my ( $self, $msg ) = @_;
+    
+    if ( defined $msg ) {
+        $self->_user_error_msg( $msg );
+        
+        # Make it look like I'm calling error() from the caller, instead of this
+        # function.
 
-    if (defined $msg) {
-        $self->_user_error_msg($msg);
-        error($msg);
+        my ( $package, $filename, $line, $sub ) = caller( 1 );
+        error( 
+            { 
+                caller_package  => '',
+                caller_filename => $filename,
+                caller_line     => $line,
+                caller_sub      => $sub,
+                caller_depth_modifier => 1,
+            }, 
+            $msg
+        );
     }
-
+    
     return $self->_user_error_msg;
 }
 
@@ -336,28 +362,27 @@ Confirms that the object is valid.  Checks that required attributes are set.
 
 =cut
 
-sub validate {
+sub validate
+{
     trace '()';
-    my ($self) = shift;
-
-    my @required = $self->get_grouped_attrs('Required');
-    my @optional = $self->get_grouped_attrs('Optional');
-
-    info("required = " . join(', ', @required));
-    trace("optional = " . join(', ', @optional));
-
+    my ( $self ) = shift;
+    
+    my @required = $self->get_grouped_attrs( 'Required' );
+    my @optional = $self->get_grouped_attrs( 'Optional' );
+    
+    debug(  "required = " . join (', ', @required ) ); 
+    debug3( "optional = " . join (', ', @optional ) );    
+    
     my @missing;
-    foreach my $required_field (@required) {
-        if (!$self->$required_field()) {
+    foreach my $required_field ( @required ) {
+        if ( ! $self->$required_field() ) {
             push @missing, $required_field;
         }
     }
-
-    if (@missing) {
-        my $user_error = "Missing required argument(s): " . join ", ",
-            @missing;
-        $self->user_error($user_error);
-        $self->invalid(1);
+    
+    if ( @missing ) {
+        $self->user_error( "Missing required argument(s): " . join ", ", @missing );
+        $self->invalid( 1 );
         return 0;
     }
     else {
@@ -369,12 +394,16 @@ sub validate {
 
 =cut
 
-# attr_name = Attribute Name.
-sub get_grouped_attrs {
-    my ($self, $attr_name) = @_;
+sub get_grouped_attrs
+{
+    my ( $self, $attr_name ) = @_;
+    
+    # attr_name = Attribute Name.
+    
     my @results = $self->$attr_name();
-
-   #print "get_grouped_attrs( $attr_name ): " . join( ', ', @results ) . "\n";
+    
+    #print "get_grouped_attrs( $attr_name ): " . join( ', ', @results ) . "\n";
+    
     return @results;
 }
 
@@ -402,8 +431,7 @@ The origin zipcode.
 
 =item * from_state
 
-The origin state in two-letter code format or full-name format.  Required for 
-UPS_Offline.
+The origin state in two-letter code format or full-name format.  Required for UPS_Offline.
 
 =item * to_zip
 
@@ -437,56 +465,46 @@ this, while UPS_Offline does not.
 
 =cut
 
-sub rate_request {
-    my $class   = shift;
-    my (%opt)   = @_;
-    my $shipper = $opt{shipper};
-
-    Carp::croak 'shipper required' unless $opt{shipper};
-
-    $shipper = _compat_shipper_name($shipper);
-
-    my $rr = Business::Shipping->_new_subclass($shipper . '::RateRequest');
-    logdie "New $shipper::RateRequest object was undefined."
-        if not defined $rr;
-
-    $rr->init(%opt);
-
-    return $rr;
-}
-
-=head2 _compat_shipper_name
-
-Shipper name backwards-compatibility
-
-1. Really old: "UPS" or "USPS" (assumes Online::)
-2. Semi-old:   "Online::UPS", "Offline::UPS", or "Online::USPS"
-3. Current:    "UPS_Online", "UPS_Offline", or "USPS_Online"
-
-=cut
-
-sub _compat_shipper_name {
-    my ($shipper) = @_;
+sub rate_request
+{
+    my $class = shift;
+    my ( %opt ) = @_;
+    my $shipper = $opt{ shipper };
+    
+    Carp::croak 'shipper required' unless $opt{ shipper };
 
+    # COMPAT: shipper compatibility
+    # 1. Really old: "UPS" or "USPS" (assumes Online::)
+    # 2. Semi-old:   "Online::UPS", "Offline::UPS", or "Online::USPS"
+    # 3. Current:    "UPS_Online", "UPS_Offline", or "USPS_Online"
+    
     my %old_to_new = (
         'Online::UPS'  => 'UPS_Online',
         'Offline::UPS' => 'UPS_Offline',
         'Online::USPS' => 'USPS_Online',
-        'UPS'          => 'UPS_Online',
-        'USPS'         => 'USPS_Online'
+        'UPS'  => 'UPS_Online',
+        'USPS' => 'USPS_Online'
     );
-    $shipper = $old_to_new{$shipper} if $old_to_new{$shipper};
-
-    return $shipper;
+    
+    $shipper = $old_to_new{ $shipper } if $old_to_new{ $shipper };
+    
+    # /COMPAT    
+    
+    
+    my $rr = Business::Shipping->_new_subclass( $shipper . '::RateRequest' );
+    logdie "New $shipper::RateRequest object was undefined." if not defined $rr;
+    
+    $rr->init( %opt );
+   
+    return $rr;
 }
 
 =head2 Business::Shipping->log_level()
 
-Simple alternative to editing the config/log4perl.conf file.  Sets the log 
-level for all Business::Shipping objects.  
+Simple alternative to editing the config/log4perl.conf file.  Sets the log level
+for all Business::Shipping objects.  
 
-Takes a scalar that can be 'trace', 'debug', 'info', 'warn', 'error', or 
-'fatal'.
+Takes a scalar that can be 'debug', 'info', 'warn', 'error', or 'fatal'.
 
 =cut
 
@@ -496,29 +514,63 @@ Takes a scalar that can be 'trace', 'debug', 'info', 'warn', 'error', or
 #
 #Private Method.
 #
-#Generates an object of a given subclass dynamically.  Will dynamically 'use'
-#the corresponding module, unless runtime module loading has been disabled via
+#Generates an object of a given subclass dynamically.  Will dynamically 'use' 
+#the corresponding module, unless runtime module loading has been disabled via 
 #the 'preload' option.
 #
 #=cut
 
-sub _new_subclass {
-    my ($class, $subclass, %opt) = @_;
-
-    croak("Error before _new_subclass was called: $@") if $@;
-
+sub _new_subclass
+{
+    my ( $class, $subclass, %opt ) = @_;
+    
+    Carp::croak( "Error before _new_subclass was called: $@" ) if $@;
+    
     my $new_class = $class . '::' . $subclass;
-
-    if ($Business::Shipping::RuntimeLoad) {
-        eval "use $new_class";
-    }
-
-    croak("Error when trying to use $new_class: \n\t$@") if $@;
-
+    
+    if ( $Business::Shipping::RuntimeLoad )
+        { eval "use $new_class"; }
+        
+    Carp::croak( "Error when trying to use $new_class: \n\t$@" ) if $@;
+    
     my $new_sub_object = eval "$new_class->new()";
-    croak("Failed to create new $new_class object.  Error: $@") if $@;
+    Carp::croak( "Failed to create new $new_class object.  Error: $@" ) if $@;
+    
+    return $new_sub_object;    
+}
 
-    return $new_sub_object;
+# COMPAT: event_handlers()
+
+#=head2 $obj->event_handlers()
+#
+#For backwards compatibility with 1.06 and prior only.
+#
+#=cut
+
+sub event_handlers
+{
+    my ( $self, $event_handlers_hash ) = @_;
+    
+    KEY: foreach my $key ( keys %$event_handlers_hash ) {
+        $key = uc $key;
+        foreach my $Log_Level ( @Business::Shipping::KLogging::Levels ) {
+            if ( $key eq $Log_Level ) {
+                # We ignore the value of the key (whether STDERR, STDOUT, etc.),
+                # because it would be a lot of work to set it up correctly, and 
+                # if a user is going to use the debug system, they would probably
+                # be willing to upgrade to the most recent version.
+                
+                # The levels are in order from least to greatest, so as soon as 
+                # we get a match, we need to stop, because the lowest level (DEBUG)
+                # will automatically include all of the greater levels.
+                
+                Business::Shipping->log_level( $Log_Level );
+                last KEY;
+            }
+        }
+    }
+    
+    return;
 }
 
 sub Optional { return qw/ tx_type /; }
@@ -537,7 +589,8 @@ Important modules that are related to Business::Shipping:
 
 =item * Business::Shipping::DataFiles - Required for offline cost estimation
 
-=item * Business::Shipping::DataTools - Tools that generate DataFiles (optional)
+=item * Business::Shipping::DataTools - Tools that generate DataFiles 
+        (optional)
 
 =back
 
@@ -551,8 +604,6 @@ very few prerequisites.  Supports shipments that originate in USA and Canada.
 =item * Business::UPS - Online cost estimation module that uses the UPS web form
 instead of the UPS Online Tools.  For shipments that originate in the USA only.
 
-=item * Net::UPS - Implementation of UPS Online Tools API in Perl
-
 =item * http://www.halofree.com/lib/public/code/Ship/UPS.pm
 
 =item * http://www.halofree.com/lib/public/code/Ship/USPS.pm
@@ -561,13 +612,11 @@ instead of the UPS Online Tools.  For shipments that originate in the USA only.
  
 =head1 Use of this software
 
-Please let the author know how you are using Business::Shipping.
+It is appreciated when users mention their use of Business::Shipping to the 
+author and/or on their website or in their application.
 
 =over 4
 
-=item * Advanced support and integration services are available from the 
-author.
-
 =item * Interchange e-commerce system ( L<http://www.icdevgroup.org> ).  See 
     C<UserTag/business-shipping.tag>.
 
@@ -578,7 +627,7 @@ author.
 =item * The "Shopping Cart" Wobject for the WebGUI project, by Andy Grundman 
     L<http://www.plainblack.com/shopping_cart_wobject>
 
-=item * Mentioned in YAPC 2004: "Writing web applications with perl ..."
+=item * Mentioned in YAPC 2004 Presentation: "Writing web applications with perl ..."
 
 =back
 
@@ -592,9 +641,9 @@ Backpan (old releases): L<http://backpan.cpan.org/authors/id/D/DB/DBROWNING/>
 
 =head1 SUPPORT
 
-This module is supported by the author. Please report any bugs or feature 
+This module is supported by the author.  Please report any bugs or feature 
 requests to C<bug-business-shipping@rt.cpan.org>, or through the web interface 
-at L<http://rt.cpan.org>. The author will be notified, and then you'll 
+at L<http://rt.cpan.org>.  The author will be notified, and then you'll 
 automatically be notified of progress on your bug as the author makes changes.
 
 =head1 KNOWN BUGS
@@ -603,16 +652,16 @@ See the C<doc/Todo> file for a comprehensive list of known bugs.
 
 =head1 CREDITS
 
-Many people have contributed to this software, please see the C<CREDITS> file. 
+Many people have contributed to this module, please see the C<doc/Credits> file. 
 
 =head1 AUTHOR
 
-Daniel Browning, db@kavod.com, L<http://www.kavod.com/>
+Daniel Browning E<lt>F<db@kavod.com>E<gt>, Kavod Technologies, L<http://www.kavod.com>.
 
 =head1 COPYRIGHT AND LICENCE
 
-Copyright 2003-2011 Daniel Browning <db@kavod.com>. All rights reserved.
-This program is free software; you may redistribute it and/or modify it 
-under the same terms as Perl itself. See LICENSE for more info.
+Copyright (c) 2003-2005 Daniel Browning E<lt>F<db@kavod.com>E<gt>. All rights 
+reserved.  This program is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.  See C<doc/License> for more info.
 
 =cut
